@@ -3,6 +3,8 @@ package nghttp2
 import (
 	"bufio"
 	"bytes"
+	"cmp"
+	"context"
 	"crypto/tls"
 	"encoding/binary"
 	"errors"
@@ -14,7 +16,7 @@ import (
 	"net/url"
 	"os"
 	"os/exec"
-	"sort"
+	"slices"
 	"strconv"
 	"strings"
 	"syscall"
@@ -42,7 +44,6 @@ func pair(name, value string) hpack.HeaderField {
 }
 
 type serverTester struct {
-	args          []string  // command-line arguments
 	cmd           *exec.Cmd // test frontend server process, which is test subject
 	url           string    // test frontend server URL
 	t             *testing.T
@@ -76,21 +77,35 @@ type options struct {
 	// tlsConfig is the client side TLS configuration that is used
 	// when tls is true.
 	tlsConfig *tls.Config
+	// tcpData is additional data that are written to connection
+	// before TLS handshake starts.  This field is ignored if tls
+	// is false.
+	tcpData []byte
+	// quic, if set to true, sets up QUIC frontend connection.
+	// quic implies tls = true.
+	quic bool
 }
 
 // newServerTester creates test context.
 func newServerTester(t *testing.T, opts options) *serverTester {
+	if opts.quic {
+		opts.tls = true
+	}
+
 	if opts.handler == nil {
 		opts.handler = noopHandler
 	}
+
 	if opts.connectPort == 0 {
 		opts.connectPort = serverPort
 	}
 
 	ts := httptest.NewUnstartedServer(opts.handler)
 
-	var args []string
-	var backendTLS, dns, externalDNS, acceptProxyProtocol, redirectIfNotTLS, affinityCookie, alpnH1 bool
+	var (
+		args                                                                                        []string
+		backendTLS, dns, externalDNS, acceptProxyProtocol, redirectIfNotTLS, affinityCookie, alpnH1 bool
+	)
 
 	for _, k := range opts.args {
 		switch k {
@@ -113,6 +128,7 @@ func newServerTester(t *testing.T, opts options) *serverTester {
 			args = append(args, k)
 		}
 	}
+
 	if backendTLS {
 		nghttp2.ConfigureServer(ts.Config, &nghttp2.Server{})
 		// According to httptest/server.go, we have to set
@@ -121,13 +137,17 @@ func newServerTester(t *testing.T, opts options) *serverTester {
 		ts.TLS = new(tls.Config)
 		ts.TLS.NextProtos = append(ts.TLS.NextProtos, "h2")
 		ts.StartTLS()
+
 		args = append(args, "-k")
 	} else {
 		ts.Start()
 	}
+
 	scheme := "http"
+
 	if opts.tls {
 		scheme = "https"
+
 		args = append(args, testDir+"/server.key", testDir+"/server.crt")
 	}
 
@@ -139,20 +159,25 @@ func newServerTester(t *testing.T, opts options) *serverTester {
 	// URL.Host looks like "127.0.0.1:8080", but we want
 	// "127.0.0.1,8080"
 	b := "-b"
+
 	if !externalDNS {
-		b += fmt.Sprintf("%v;", strings.Replace(backendURL.Host, ":", ",", -1))
+		b += fmt.Sprintf("%v;", strings.ReplaceAll(backendURL.Host, ":", ","))
 	} else {
 		sep := strings.LastIndex(backendURL.Host, ":")
 		if sep == -1 {
 			t.Fatalf("backendURL.Host %v does not contain separator ':'", backendURL.Host)
 		}
+
 		// We use awesome service nip.io.
 		b += fmt.Sprintf("%v.nip.io,%v;", backendURL.Host[:sep], backendURL.Host[sep+1:])
+		// Make external DNS tests less flaky.
+		args = append(args, "--backend-address-family=IPv4")
 	}
 
 	if backendTLS {
 		b += ";proto=h2;tls"
 	}
+
 	if dns {
 		b += ";dns"
 	}
@@ -166,11 +191,13 @@ func newServerTester(t *testing.T, opts options) *serverTester {
 	}
 
 	noTLS := ";no-tls"
+
 	if opts.tls {
 		noTLS = ""
 	}
 
 	var proxyProto string
+
 	if acceptProxyProtocol {
 		proxyProto = ";proxyproto"
 	}
@@ -178,6 +205,24 @@ func newServerTester(t *testing.T, opts options) *serverTester {
 	args = append(args, fmt.Sprintf("-f127.0.0.1,%v%v%v", serverPort, noTLS, proxyProto), b,
 		"--errorlog-file="+logDir+"/log.txt", "-LINFO")
 
+	if opts.quic {
+		args = append(args,
+			fmt.Sprintf("-f127.0.0.1,%v;quic", serverPort),
+			"--no-quic-bpf",
+			// quic-go client just closes connection after
+			// receiving the first GOAWAY without any
+			// indication like sending CONNECTION_CLOSE if
+			// there is no active stream.  If that
+			// happens, server keeps resending 2nd GOAWAY
+			// until idle timeout passes.  If that happens
+			// during Close(), the process will be killed
+			// because the default idle timeout is longer
+			// than the close timeout.  Shorten the idle
+			// timeout to prevent it from happening.
+			"--frontend-quic-idle-timeout=5",
+		)
+	}
+
 	authority := fmt.Sprintf("127.0.0.1:%v", opts.connectPort)
 
 	st := &serverTester{
@@ -201,45 +246,55 @@ func newServerTester(t *testing.T, opts options) *serverTester {
 	}
 
 	retry := 0
+
 	for {
 		time.Sleep(50 * time.Millisecond)
 
-		var conn net.Conn
-		var err error
-		if opts.tls {
+		conn, err := net.Dial("tcp", authority)
+		if err == nil && opts.tls {
+			if len(opts.tcpData) > 0 {
+				if _, err := conn.Write(opts.tcpData); err != nil {
+					st.Close()
+					st.t.Fatal("Error writing TCP data")
+				}
+			}
+
 			var tlsConfig *tls.Config
+
 			if opts.tlsConfig == nil {
 				tlsConfig = new(tls.Config)
 			} else {
 				tlsConfig = opts.tlsConfig.Clone()
 			}
+
 			tlsConfig.InsecureSkipVerify = true
+
 			if alpnH1 {
 				tlsConfig.NextProtos = []string{"http/1.1"}
 			} else {
 				tlsConfig.NextProtos = []string{"h2"}
 			}
-			conn, err = tls.Dial("tcp", authority, tlsConfig)
-		} else {
-			conn, err = net.Dial("tcp", authority)
+
+			tlsConn := tls.Client(conn, tlsConfig)
+
+			err = tlsConn.Handshake()
+			if err == nil {
+				conn = tlsConn
+			}
 		}
+
 		if err != nil {
-			retry += 1
+			retry++
 			if retry >= 100 {
 				st.Close()
 				st.t.Fatalf("Error server is not responding too long; server command-line arguments may be invalid")
 			}
+
 			continue
 		}
-		if opts.tls {
-			tlsConn := conn.(*tls.Conn)
-			cs := tlsConn.ConnectionState()
-			if !cs.NegotiatedProtocolIsMutual {
-				st.Close()
-				st.t.Fatalf("Error negotiated next protocol is not mutual")
-			}
-		}
+
 		st.conn = conn
+
 		break
 	}
 
@@ -256,22 +311,33 @@ func (st *serverTester) Close() {
 	if st.conn != nil {
 		st.conn.Close()
 	}
+
 	if st.cmd != nil {
 		done := make(chan struct{})
+
 		go func() {
-			st.cmd.Wait()
+			if err := st.cmd.Wait(); err != nil {
+				st.t.Errorf("Error st.cmd.Wait() = %v", err)
+			}
+
 			close(done)
 		}()
 
-		st.cmd.Process.Signal(syscall.SIGQUIT)
+		if err := st.cmd.Process.Signal(syscall.SIGQUIT); err != nil {
+			st.t.Errorf("Error st.cmd.Process.Signal() = %v", err)
+		}
 
 		select {
 		case <-done:
 		case <-time.After(10 * time.Second):
-			st.cmd.Process.Kill()
+			if err := st.cmd.Process.Kill(); err != nil {
+				st.t.Errorf("Error st.cmd.Process.Kill() = %v", err)
+			}
+
 			<-done
 		}
 	}
+
 	if st.ts != nil {
 		st.ts.Close()
 	}
@@ -284,6 +350,7 @@ func (st *serverTester) readFrame() (http2.Frame, error) {
 			st.errCh <- err
 			return
 		}
+
 		st.frCh <- f
 	}()
 
@@ -324,14 +391,16 @@ func (cbr *chunkedBodyReader) Read(p []byte) (n int, err error) {
 	// after request was sent and before body returns EOF.
 	if !cbr.trailerWritten {
 		cbr.trailerWritten = true
+
 		for _, h := range cbr.trailer {
 			cbr.req.Trailer.Set(h.Name, h.Value)
 		}
 	}
+
 	return cbr.body.Read(p)
 }
 
-func (st *serverTester) websocket(rp requestParam) (*serverResponse, error) {
+func (st *serverTester) websocket(rp requestParam) *serverResponse {
 	urlstring := st.url + "/echo"
 
 	config, err := websocket.NewConfig(urlstring, st.url)
@@ -340,6 +409,7 @@ func (st *serverTester) websocket(rp requestParam) (*serverResponse, error) {
 	}
 
 	config.Header.Add("Test-Case", rp.name)
+
 	for _, h := range rp.header {
 		config.Header.Add(h.Name, h.Value)
 	}
@@ -354,7 +424,9 @@ func (st *serverTester) websocket(rp requestParam) (*serverResponse, error) {
 	}
 
 	msg := make([]byte, 1024)
+
 	var n int
+
 	if n, err = ws.Read(msg); err != nil {
 		st.t.Fatalf("ws.Read() returned error: %v", err)
 	}
@@ -363,24 +435,30 @@ func (st *serverTester) websocket(rp requestParam) (*serverResponse, error) {
 		body: msg[:n],
 	}
 
-	return res, nil
+	return res
 }
 
 func (st *serverTester) http1(rp requestParam) (*serverResponse, error) {
 	method := "GET"
+
 	if rp.method != "" {
 		method = rp.method
 	}
 
-	var body io.Reader
-	var cbr *chunkedBodyReader
+	var (
+		body io.Reader
+		cbr  *chunkedBodyReader
+	)
+
 	if rp.body != nil {
 		body = bytes.NewBuffer(rp.body)
+
 		if len(rp.trailer) != 0 {
 			cbr = &chunkedBodyReader{
 				trailer: rp.trailer,
 				body:    body,
 			}
+
 			body = cbr
 		}
 	}
@@ -392,39 +470,51 @@ func (st *serverTester) http1(rp requestParam) (*serverResponse, error) {
 		if err != nil {
 			st.t.Fatalf("Error parsing URL from st.url %v: %v", st.url, err)
 		}
+
 		u.Path = ""
 		u.RawQuery = ""
 		reqURL = u.String() + rp.path
 	}
 
-	req, err := http.NewRequest(method, reqURL, body)
+	ctx, cancel := context.WithTimeout(context.Background(), 30*time.Second)
+	defer cancel()
+
+	req, err := http.NewRequestWithContext(ctx, method, reqURL, body)
 	if err != nil {
 		return nil, err
 	}
+
 	for _, h := range rp.header {
 		req.Header.Add(h.Name, h.Value)
 	}
+
 	req.Header.Add("Test-Case", rp.name)
+
 	if cbr != nil {
 		cbr.req = req
 		// this makes request use chunked encoding
 		req.ContentLength = -1
 		req.Trailer = make(http.Header)
+
 		for _, h := range cbr.trailer {
 			req.Trailer.Set(h.Name, "")
 		}
 	}
+
 	if err := req.Write(st.conn); err != nil {
 		return nil, err
 	}
+
 	resp, err := http.ReadResponse(bufio.NewReader(st.conn), req)
 	if err != nil {
 		return nil, err
 	}
+
 	respBody, err := io.ReadAll(resp.Body)
 	if err != nil {
 		return nil, err
 	}
+
 	resp.Body.Close()
 
 	res := &serverResponse{
@@ -442,6 +532,7 @@ func (st *serverTester) http2(rp requestParam) (*serverResponse, error) {
 	st.header = make(http.Header)
 
 	var id uint32
+
 	if rp.streamID != 0 {
 		id = rp.streamID
 		if id >= st.nextStreamID && id%2 == 1 {
@@ -455,6 +546,7 @@ func (st *serverTester) http2(rp requestParam) (*serverResponse, error) {
 	if !st.h2PrefaceSent {
 		st.h2PrefaceSent = true
 		fmt.Fprint(st.conn, "PRI * HTTP/2.0\r\n\r\nSM\r\n\r\n")
+
 		if err := st.fr.WriteSettings(); err != nil {
 			return nil, err
 		}
@@ -472,26 +564,32 @@ func (st *serverTester) http2(rp requestParam) (*serverResponse, error) {
 		if rp.method != "" {
 			method = rp.method
 		}
+
 		_ = st.enc.WriteField(pair(":method", method))
 
 		scheme := "http"
+
 		if rp.scheme != "" {
 			scheme = rp.scheme
 		}
+
 		_ = st.enc.WriteField(pair(":scheme", scheme))
 
 		authority := st.authority
+
 		if rp.authority != "" {
 			authority = rp.authority
 		}
+
 		_ = st.enc.WriteField(pair(":authority", authority))
 
 		path := "/"
+
 		if rp.path != "" {
 			path = rp.path
 		}
-		_ = st.enc.WriteField(pair(":path", path))
 
+		_ = st.enc.WriteField(pair(":path", path))
 		_ = st.enc.WriteField(pair("test-case", rp.name))
 
 		for _, h := range rp.header {
@@ -517,9 +615,11 @@ func (st *serverTester) http2(rp requestParam) (*serverResponse, error) {
 
 		if len(rp.trailer) != 0 {
 			st.headerBlkBuf.Reset()
+
 			for _, h := range rp.trailer {
 				_ = st.enc.WriteField(h)
 			}
+
 			err := st.fr.WriteHeaders(http2.HeadersFrameParam{
 				StreamID:      id,
 				EndStream:     true,
@@ -537,56 +637,65 @@ loop:
 		if err != nil {
 			return res, err
 		}
+
 		switch f := fr.(type) {
 		case *http2.HeadersFrame:
 			_, err := st.dec.Write(f.HeaderBlockFragment())
 			if err != nil {
 				return res, err
 			}
-			sr, ok := streams[f.FrameHeader.StreamID]
+
+			sr, ok := streams[f.StreamID]
 			if !ok {
 				st.header = make(http.Header)
 				break
 			}
+
 			sr.header = cloneHeader(st.header)
+
 			var status int
+
 			status, err = strconv.Atoi(sr.header.Get(":status"))
 			if err != nil {
-				return res, fmt.Errorf("Error parsing status code: %w", err)
+				return res, fmt.Errorf("could not parse :status: %w", err)
 			}
+
 			sr.status = status
-			if f.StreamEnded() {
-				if streamEnded(res, streams, sr) {
-					break loop
-				}
+
+			if f.StreamEnded() && streamEnded(res, streams, sr) {
+				break loop
 			}
 		case *http2.PushPromiseFrame:
 			_, err := st.dec.Write(f.HeaderBlockFragment())
 			if err != nil {
 				return res, err
 			}
+
 			sr := &serverResponse{
 				streamID:  f.PromiseID,
 				reqHeader: cloneHeader(st.header),
 			}
+
 			streams[sr.streamID] = sr
 		case *http2.DataFrame:
-			sr, ok := streams[f.FrameHeader.StreamID]
+			sr, ok := streams[f.StreamID]
 			if !ok {
 				break
 			}
+
 			sr.body = append(sr.body, f.Data()...)
-			if f.StreamEnded() {
-				if streamEnded(res, streams, sr) {
-					break loop
-				}
+
+			if f.StreamEnded() && streamEnded(res, streams, sr) {
+				break loop
 			}
 		case *http2.RSTStreamFrame:
-			sr, ok := streams[f.FrameHeader.StreamID]
+			sr, ok := streams[f.StreamID]
 			if !ok {
 				break
 			}
+
 			sr.errCode = f.ErrCode
+
 			if streamEnded(res, streams, sr) {
 				break loop
 			}
@@ -594,27 +703,36 @@ loop:
 			if f.ErrCode == http2.ErrCodeNo {
 				break
 			}
+
 			res.errCode = f.ErrCode
 			res.connErr = true
+
 			break loop
 		case *http2.SettingsFrame:
 			if f.IsAck() {
 				break
 			}
+
 			if err := st.fr.WriteSettingsAck(); err != nil {
 				return res, err
 			}
 		}
 	}
-	sort.Sort(ByStreamID(res.pushResponse))
+
+	slices.SortFunc(res.pushResponse, func(a, b *serverResponse) int {
+		return cmp.Compare(a.streamID, b.streamID)
+	})
+
 	return res, nil
 }
 
 func streamEnded(mainSr *serverResponse, streams map[uint32]*serverResponse, sr *serverResponse) bool {
 	delete(streams, sr.streamID)
+
 	if mainSr.streamID != sr.streamID {
 		mainSr.pushResponse = append(mainSr.pushResponse, sr)
 	}
+
 	return len(streams) == 0
 }
 
@@ -626,36 +744,26 @@ type serverResponse struct {
 	errCode      http2.ErrCode     // error code received in HTTP/2 RST_STREAM or GOAWAY
 	connErr      bool              // true if HTTP/2 connection error
 	connClose    bool              // Connection: close is included in response header in HTTP/1 test
-	reqHeader    http.Header       // http request header, currently only sotres pushed request header
+	reqHeader    http.Header       // http request header, currently only stores pushed request header
 	pushResponse []*serverResponse // pushed response
 }
 
-type ByStreamID []*serverResponse
-
-func (b ByStreamID) Len() int {
-	return len(b)
-}
-
-func (b ByStreamID) Swap(i, j int) {
-	b[i], b[j] = b[j], b[i]
-}
-
-func (b ByStreamID) Less(i, j int) bool {
-	return b[i].streamID < b[j].streamID
-}
-
 func cloneHeader(h http.Header) http.Header {
 	h2 := make(http.Header, len(h))
+
 	for k, vv := range h {
 		vv2 := make([]string, len(vv))
 		copy(vv2, vv)
 		h2[k] = vv2
 	}
+
 	return h2
 }
 
 func noopHandler(w http.ResponseWriter, r *http.Request) {
-	io.ReadAll(r.Body)
+	if _, err := io.ReadAll(r.Body); err != nil {
+		http.Error(w, fmt.Sprintf("Error io.ReadAll() = %v", err), http.StatusInternalServerError)
+	}
 }
 
 type APIResponse struct {
@@ -695,39 +803,71 @@ const (
 	proxyProtocolV2ProtocolDgram  proxyProtocolV2Protocol = 0x2
 )
 
-func writeProxyProtocolV2(w io.Writer, hdr proxyProtocolV2) {
-	w.Write([]byte{0x0D, 0x0A, 0x0D, 0x0A, 0x00, 0x0D, 0x0A, 0x51, 0x55, 0x49, 0x54, 0x0A})
-	w.Write([]byte{byte(0x20 | hdr.command)})
+func writeProxyProtocolV2(w io.Writer, hdr proxyProtocolV2) error {
+	if _, err := w.Write([]byte{0x0D, 0x0A, 0x0D, 0x0A, 0x00, 0x0D, 0x0A, 0x51, 0x55, 0x49, 0x54, 0x0A}); err != nil {
+		return err
+	}
+
+	if _, err := w.Write([]byte{byte(0x20 | hdr.command)}); err != nil {
+		return err
+	}
 
 	switch srcAddr := hdr.sourceAddress.(type) {
 	case *net.TCPAddr:
 		dstAddr := hdr.destinationAddress.(*net.TCPAddr)
+
 		if len(srcAddr.IP) != len(dstAddr.IP) {
 			panic("len(srcAddr.IP) != len(dstAddr.IP)")
 		}
+
 		var fam byte
+
 		if len(srcAddr.IP) == 4 {
 			fam = byte(proxyProtocolV2FamilyInet << 4)
 		} else {
 			fam = byte(proxyProtocolV2FamilyInet6 << 4)
 		}
+
 		fam |= byte(proxyProtocolV2ProtocolStream)
-		w.Write([]byte{fam})
+
+		if _, err := w.Write([]byte{fam}); err != nil {
+			return err
+		}
+
 		length := uint16(len(srcAddr.IP)*2 + 4 + len(hdr.additionalData))
-		binary.Write(w, binary.BigEndian, length)
-		w.Write(srcAddr.IP)
-		w.Write(dstAddr.IP)
-		binary.Write(w, binary.BigEndian, uint16(srcAddr.Port))
-		binary.Write(w, binary.BigEndian, uint16(dstAddr.Port))
+
+		if err := binary.Write(w, binary.BigEndian, length); err != nil {
+			return err
+		}
+
+		if _, err := w.Write(srcAddr.IP); err != nil {
+			return err
+		}
+
+		if _, err := w.Write(dstAddr.IP); err != nil {
+			return err
+		}
+
+		if err := binary.Write(w, binary.BigEndian, uint16(srcAddr.Port)); err != nil {
+			return err
+		}
+
+		if err := binary.Write(w, binary.BigEndian, uint16(dstAddr.Port)); err != nil {
+			return err
+		}
 	case *net.UnixAddr:
 		dstAddr := hdr.destinationAddress.(*net.UnixAddr)
+
 		if len(srcAddr.Name) > 108 {
 			panic("too long Unix source address")
 		}
+
 		if len(dstAddr.Name) > 108 {
 			panic("too long Unix destination address")
 		}
+
 		fam := byte(proxyProtocolV2FamilyUnix << 4)
+
 		switch srcAddr.Net {
 		case "unix":
 			fam |= byte(proxyProtocolV2ProtocolStream)
@@ -736,20 +876,51 @@ func writeProxyProtocolV2(w io.Writer, hdr proxyProtocolV2) {
 		default:
 			fam |= byte(proxyProtocolV2ProtocolUnspec)
 		}
-		w.Write([]byte{fam})
+
+		if _, err := w.Write([]byte{fam}); err != nil {
+			return err
+		}
+
 		length := uint16(216 + len(hdr.additionalData))
-		binary.Write(w, binary.BigEndian, length)
+
+		if err := binary.Write(w, binary.BigEndian, length); err != nil {
+			return err
+		}
+
 		zeros := make([]byte, 108)
-		w.Write([]byte(srcAddr.Name))
-		w.Write(zeros[:108-len(srcAddr.Name)])
-		w.Write([]byte(dstAddr.Name))
-		w.Write(zeros[:108-len(dstAddr.Name)])
+
+		if _, err := w.Write([]byte(srcAddr.Name)); err != nil {
+			return err
+		}
+
+		if _, err := w.Write(zeros[:108-len(srcAddr.Name)]); err != nil {
+			return err
+		}
+
+		if _, err := w.Write([]byte(dstAddr.Name)); err != nil {
+			return err
+		}
+
+		if _, err := w.Write(zeros[:108-len(dstAddr.Name)]); err != nil {
+			return err
+		}
 	default:
 		fam := byte(proxyProtocolV2FamilyUnspec<<4) | byte(proxyProtocolV2ProtocolUnspec)
-		w.Write([]byte{fam})
+
+		if _, err := w.Write([]byte{fam}); err != nil {
+			return err
+		}
+
 		length := uint16(len(hdr.additionalData))
-		binary.Write(w, binary.BigEndian, length)
+
+		if err := binary.Write(w, binary.BigEndian, length); err != nil {
+			return err
+		}
 	}
 
-	w.Write(hdr.additionalData)
+	if _, err := w.Write(hdr.additionalData); err != nil {
+		return err
+	}
+
+	return nil
 }
