@@ -743,13 +743,15 @@ int Worker::create_tcp_server_socket(UpstreamAddr &faddr) {
   auto &listenerconf = get_config()->conn.listener;
 
   auto service = util::utos(faddr.port);
-  addrinfo hints{};
-  hints.ai_family = faddr.family;
-  hints.ai_socktype = SOCK_STREAM;
-  hints.ai_flags = AI_PASSIVE;
+  addrinfo hints{
+    .ai_flags = AI_PASSIVE
 #ifdef AI_ADDRCONFIG
-  hints.ai_flags |= AI_ADDRCONFIG;
+                | AI_ADDRCONFIG
 #endif // AI_ADDRCONFIG
+    ,
+    .ai_family = faddr.family,
+    .ai_socktype = SOCK_STREAM,
+  };
 
   auto node = faddr.host == "*"sv ? nullptr : faddr.host.data();
 
@@ -1085,13 +1087,15 @@ int Worker::create_quic_server_socket(UpstreamAddr &faddr) {
   int rv;
 
   auto service = util::utos(faddr.port);
-  addrinfo hints{};
-  hints.ai_family = faddr.family;
-  hints.ai_socktype = SOCK_DGRAM;
-  hints.ai_flags = AI_PASSIVE;
+  addrinfo hints{
+    .ai_flags = AI_PASSIVE
 #  ifdef AI_ADDRCONFIG
-  hints.ai_flags |= AI_ADDRCONFIG;
+                | AI_ADDRCONFIG
 #  endif // AI_ADDRCONFIG
+    ,
+    .ai_family = faddr.family,
+    .ai_socktype = SOCK_DGRAM,
+  };
 
   auto node = faddr.host == "*"sv ? nullptr : faddr.host.data();
 
@@ -1194,8 +1198,8 @@ int Worker::create_quic_server_socket(UpstreamAddr &faddr) {
         continue;
       }
 
-#  if defined(IPV6_MTU_DISCOVER) && defined(IPV6_PMTUDISC_DO)
-      int mtu_disc = IPV6_PMTUDISC_DO;
+#  if defined(IPV6_MTU_DISCOVER) && defined(IPV6_PMTUDISC_PROBE)
+      int mtu_disc = IPV6_PMTUDISC_PROBE;
       if (setsockopt(fd, IPPROTO_IPV6, IPV6_MTU_DISCOVER, &mtu_disc,
                      static_cast<socklen_t>(sizeof(mtu_disc))) == -1) {
         auto error = errno;
@@ -1205,7 +1209,7 @@ int Worker::create_quic_server_socket(UpstreamAddr &faddr) {
         close(fd);
         continue;
       }
-#  endif // defined(IPV6_MTU_DISCOVER) && defined(IP_PMTUDISC_DO)
+#  endif // defined(IPV6_MTU_DISCOVER) && defined(IPV6_PMTUDISC_PROBE)
     } else {
       if (setsockopt(fd, IPPROTO_IP, IP_PKTINFO, &val,
                      static_cast<socklen_t>(sizeof(val))) == -1) {
@@ -1225,8 +1229,8 @@ int Worker::create_quic_server_socket(UpstreamAddr &faddr) {
         continue;
       }
 
-#  if defined(IP_MTU_DISCOVER) && defined(IP_PMTUDISC_DO)
-      int mtu_disc = IP_PMTUDISC_DO;
+#  if defined(IP_MTU_DISCOVER) && defined(IP_PMTUDISC_PROBE)
+      int mtu_disc = IP_PMTUDISC_PROBE;
       if (setsockopt(fd, IPPROTO_IP, IP_MTU_DISCOVER, &mtu_disc,
                      static_cast<socklen_t>(sizeof(mtu_disc))) == -1) {
         auto error = errno;
@@ -1235,7 +1239,7 @@ int Worker::create_quic_server_socket(UpstreamAddr &faddr) {
         close(fd);
         continue;
       }
-#  endif // defined(IP_MTU_DISCOVER) && defined(IP_PMTUDISC_DO)
+#  endif // defined(IP_MTU_DISCOVER) && defined(IP_PMTUDISC_PROBE)
     }
 
 #  ifdef UDP_GRO
